import { HeartFilled, HeartOutlined } from "@ant-design/icons";
import { Button, Flex, Form, Input, InputRef, Typography } from "antd";
import {
  useCreateLessonQuestionMutation,
  useCreateLessonQuestionReplyMutation,
  useDislikeQuestionMutation,
  useGetLessonQuestionsQuery,
  useLikeQuestionMutation,
} from "core/services/lessons";
import { LessonQuestion } from "core/types/lesson";
import { ReactNode, useEffect, useRef, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { useParams } from "react-router-dom";
import {
  likedQuestions,
  questions,
  setLikedQuestions,
  setQuestions,
} from "./lessonQuestionSlice";
import MyCenteredSpin from "shared/components/MyCenteredSpin";
import MyErrorResult from "shared/components/MyResult";
import { useMessage } from "core/context/MessageContext";
import { useForm } from "antd/es/form/Form";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import { useCachedUser } from "core/services/cachedUser";
import MyUserAvatar from "shared/components/MyUserAvatar";
import { useTranslation } from "react-i18next";

const CreateQuestionForm: React.FC = () => {
  const { t } = useTranslation();

  const { lessonId } = useParams();
  const [form] = useForm();

  const [createQuestion, { isLoading }] = useCreateLessonQuestionMutation();
  const { success, error } = useMessage();

  const handleCreateLessonQuestion = async () => {
    try {
      await createQuestion({
        lessonId: lessonId as string,
        message: form.getFieldValue("message"),
      }).unwrap();

      success(t("lessonQuestions.messageQuestionSuccessfullyCreated"));
      form.resetFields();
    } catch (err) {
      console.error(err);
      error(t("common.messageRequestFailed"));
    }
  };

  return (
    <Form
      form={form}
      layout="vertical"
      onFinish={() => handleCreateLessonQuestion()}
      requiredMark={false}
    >
      <Form.Item
        label={t("lessonQuestions.askAQuestion")}
        name="message"
        rules={[
          { required: true, message: t("lessonQuestions.ruleMessageRequired") },
        ]}
      >
        <Input.TextArea
          placeholder={t("lessonQuestions.typeSomething")}
          autoSize={{ minRows: 2 }}
          maxLength={5000}
        />
      </Form.Item>

      <Form.Item>
        <Button type="primary" htmlType="submit" loading={isLoading}>
          {t("lessonQuestions.submitButton")}
        </Button>
      </Form.Item>
    </Form>
  );
};

export default function Questions() {
  const { t } = useTranslation();

  const dispatch = useDispatch();
  const { lessonId } = useParams();

  const dataQuestions = useSelector(questions);
  const dataLikedQuestions = useSelector(likedQuestions);

  const { data, error, isLoading, refetch } = useGetLessonQuestionsQuery(
    lessonId as string,
    {
      refetchOnMountOrArgChange: true,
    }
  );

  useEffect(() => {
    if (!data) return;

    dispatch(setQuestions(data.Questions));
    dispatch(setLikedQuestions(data.LikedQuestions));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  return (
    <Flex justify="center">
      <Flex style={{ width: 800, maxWidth: 800 * 0.9 }} vertical>
        <Typography.Title level={3}>
          {t("lessonQuestions.questions")}
        </Typography.Title>

        <CreateQuestionForm />

        <Flex vertical style={{}}>
          {error ? (
            <MyErrorResult />
          ) : isLoading ? (
            <MyCenteredSpin height="100px" />
          ) : (
            dataQuestions
              .filter((question) => question.QuestionId === "")
              .sort((b, a) => a.CreatedAt.localeCompare(b.CreatedAt))
              .map((question) => (
                <QuestionItem
                  key={question.Id}
                  lessonId={lessonId as string}
                  question={question}
                  replies={dataQuestions.filter(
                    (reply) => reply.QuestionId === question.Id
                  )}
                  likedQuestions={dataLikedQuestions}
                />
              ))
          )}
        </Flex>
      </Flex>
    </Flex>
  );
}

type HandleReplyFunction = (message: string, replyId?: string) => Promise<void>;

export function QuestionItem({
  lessonId,
  question,
  replies,
  likedQuestions,
}: {
  lessonId: string;
  question: LessonQuestion;
  replies: LessonQuestion[];
  likedQuestions: string[];
}) {
  const { t } = useTranslation();
  const [showReplies, setShowReplies] = useState(1);
  const { success, error } = useMessage();

  const [reqCreateQuestionReply] = useCreateLessonQuestionReplyMutation();
  const [reqLikeQuestion] = useLikeQuestionMutation();
  const [reqDislikeQuestion] = useDislikeQuestionMutation();

  async function handleReply(message: string, replyId?: string) {
    try {
      await reqCreateQuestionReply({
        lessonId: lessonId as string,
        questionId: question.Id,
        message: message,
      }).unwrap();

      success(t("lessonQuestions.messageReplySuccessfullyCreated"));

      await new Promise((resolve) => setTimeout(resolve, 0));
    } catch (err) {
      console.error(err);
      error(t("common.messageRequestFailed"));
      await new Promise((resolve) => setTimeout(resolve, 0));
    }
  }

  async function handleLike(questionId: string) {
    try {
      await reqLikeQuestion({
        lessonId: lessonId as string,
        questionId: questionId,
      }).unwrap();
    } catch (err) {
      console.error(err);
      error(t("lessonQuestions.messageRequestFailed"));
    }
  }

  async function handleDislike(questionId: string) {
    try {
      await reqDislikeQuestion({
        lessonId: lessonId as string,
        questionId: questionId,
      }).unwrap();
    } catch (err) {
      console.error(err);
      error(t("lessonQuestions.messageRequestFailed"));
    }
  }

  return (
    <QuestionUIRaw
      userId={question.CreatorUserId}
      message={question.Message}
      childContent={
        <>
          {replies
            .sort((a, b) => a.CreatedAt.localeCompare(b.CreatedAt))
            .map((reply) => (
              <QuestionUIRaw
                key={reply.Id}
                userId={reply.CreatorUserId}
                message={reply.Message}
                childContent={<></>}
                likes={reply.Likes}
                onReply={handleReply}
                onLike={() => handleLike(reply.Id)}
                onDislike={() => handleDislike(reply.Id)}
                replyId={reply.Id}
                hasLiked={likedQuestions.some((likeId) => likeId === reply.Id)}
              />
            ))}
        </>
      }
      likes={question.Likes}
      onReply={handleReply}
      onLike={() => handleLike(question.Id)}
      onDislike={() => handleDislike(question.Id)}
      replyId={undefined}
      hasLiked={likedQuestions.some((likeId) => likeId === question.Id)}
    />
  );
}

export function QuestionUIRaw({
  userId,
  message,
  childContent,
  likes,
  replyId,
  onReply,
  onLike,
  onDislike,
  hasLiked,
}: {
  userId: string;
  message: string;
  childContent: ReactNode;
  likes: number;
  replyId?: string;
  onReply: HandleReplyFunction;
  onLike: () => void;
  onDislike: () => void;
  hasLiked: boolean;
}) {
  //const [hasLiked, setHasLiked] = useState(false);

  const { t } = useTranslation();

  const [replyFormVisible, setReplyFormVisible] = useState(false);
  const [replyMessage, setReplyMessage] = useState<null | string>(null);
  const [isSendingReply, setIsSendingReply] = useState(false);

  const { user, error, isLoading } = useCachedUser(userId);

  // useref to focus on the input field
  const inputRef = useRef<InputRef>(null);

  const userAt =
    isLoading || user === undefined
      ? ""
      : `@${user.FirstName} ${user.LastName} `;

  if (error) return <MyErrorResult />;

  if (isLoading) return <MyCenteredSpin height="80px" />;

  return (
    <>
      <Flex gap={16}>
        <MyUserAvatar
          profilePictureUrl={user.ProfilePictureUrl}
          firstName={user.FirstName}
          disableCursorPointer
        />
        <Flex vertical style={{ width: "100%" }}>
          <Typography style={{ fontSize: 24, fontWeight: 800 }}>
            {user.FirstName} {user.LastName}
          </Typography>
          <Typography style={{ fontSize: 18, fontWeight: 500 }}>
            {message}
          </Typography>
          <Flex gap={0} align="center">
            <Button
              type="text"
              icon={hasLiked ? <HeartFilled /> : <HeartOutlined />}
              shape="circle"
              size="large"
              style={{
                color: hasLiked ? "red" : undefined,
                transform: hasLiked ? "scale(1.2)" : "scale(1)",
                transition: "all 0.2s ease-in-out",
              }}
              onClick={() => (hasLiked ? onDislike() : onLike())}
            ></Button>

            <Typography
              style={{ fontSize: 16, fontWeight: 400, pointerEvents: "none" }}
            >
              {likes >= 1 ? likes : " "}
            </Typography>
            <Button
              type={replyFormVisible ? "link" : "text"}
              onClick={() => {
                if (replyMessage === null) setReplyMessage(userAt);
                setReplyFormVisible(!replyFormVisible);

                setTimeout(() => {
                  if (inputRef.current) {
                    const input = inputRef.current;
                    input.focus({ cursor: "end" });
                  }
                }, 100);
              }}
            >
              {replyFormVisible
                ? t("lessonQuestions.hide")
                : t("lessonQuestions.reply")}
            </Button>
          </Flex>
          {replyFormVisible ? (
            <Form
              initialValues={{
                reply: replyMessage ? replyMessage : userAt,
              }}
              disabled={isSendingReply}
              onFinish={async () => {
                setIsSendingReply(true);
                await onReply(replyMessage ? replyMessage : "", replyId);

                setIsSendingReply(false);
                setReplyFormVisible(false);
                setReplyMessage(null);
              }}
            >
              <Form.Item
                name="reply"
                rules={[
                  {
                    required: true,
                    message: t("lessonQuestions.ruleReplyRequired"),
                  },
                ]}
              >
                <Input.TextArea
                  ref={inputRef}
                  value={replyMessage ? replyMessage : userAt}
                  placeholder={t("lessonQuestions.ruleReplyRequired")}
                  onChange={(e) => setReplyMessage(e.target.value)}
                  autoSize={{ minRows: 2 }}
                  maxLength={5000}
                />
              </Form.Item>
              <Form.Item>
                <Button
                  type="primary"
                  loading={isSendingReply}
                  htmlType="submit"
                >
                  {t("lessonQuestions.reply")}
                </Button>
              </Form.Item>
            </Form>
          ) : null}
          {childContent}
        </Flex>
      </Flex>
    </>
  );
}
