import { PlusOutlined } from "@ant-design/icons";
import { Button, Flex, Form, Input, Select } from "antd";
import HeaderBar from "core/components/Header";
import { useMessage } from "core/context/MessageContext";
import { useCreateTeamMemberMutation } from "core/services/organization";
import { Constants, EncodeStringToBase64 } from "core/utils/utils";
import { useTranslation } from "react-i18next";
import { useNavigate } from "react-router-dom";
import MyMiddleCard from "shared/components/MyMiddleCard";

type FieldType = {
  firstName: string;
  lastName: string;
  email: string;
  changePasswordOnFirstLogin: boolean;
  password?: string;
  sendInvitationEmail: boolean;
};

export default function TeamCreateUser() {
  const { t } = useTranslation();

  const navigate = useNavigate();
  const { success } = useMessage();

  const [reqCreateTeamMember, { isLoading }] = useCreateTeamMemberMutation();

  const tmpRoleNames = t("roles.roleNames", {
    returnObjects: true,
  }) as any;

  return (
    <>
      <HeaderBar
        theme="light"
        backTo={Constants.ROUTE_PATHS.ORGANIZATION_TEAM}
      />

      <MyMiddleCard title={t("teamCreateUser.middleCardTitle")}>
        <Form
          layout="vertical"
          requiredMark={false}
          initialValues={{
            roleId: "a1f084ad-d501-4015-b326-4c5c46fd1c5e",
          }}
          onFinish={async (values) => {
            console.log(values);

            try {
              await reqCreateTeamMember({
                firstName: values.firstName,
                lastName: values.lastName,
                email: values.email,
                roleId: values.roleId,
                password: EncodeStringToBase64(values.password),
              }).unwrap();

              success(t("teamCreateUser.messageUserSuccessfullyCreated"));

              navigate(Constants.ROUTE_PATHS.ORGANIZATION_TEAM);
            } catch (error) {
              console.error(error);
            }
          }}
        >
          <Form.Item<FieldType>
            label={t("common.firstName")}
            name="firstName"
            rules={[
              { required: true, message: t("common.firstNameRules.required") },
              {
                min: Constants.GLOBALS.MIN_FIRST_NAME_LENGTH,
                message: t("common.firstNameRules.minLength", {
                  minLength: Constants.GLOBALS.MIN_FIRST_NAME_LENGTH,
                }),
              },
              {
                max: Constants.GLOBALS.MAX_FIRST_NAME_LENGTH,
                message: t("common.firstNameRules.maxLength", {
                  maxLength: Constants.GLOBALS.MAX_FIRST_NAME_LENGTH,
                }),
              },
            ]}
          >
            <Input
              placeholder={t("common.firstName")}
              maxLength={Constants.GLOBALS.MAX_FIRST_NAME_LENGTH}
            />
          </Form.Item>

          <Form.Item<FieldType>
            label={t("common.lastName")}
            name="lastName"
            rules={[
              { required: true, message: t("common.lastNameRules.required") },
              {
                min: Constants.GLOBALS.MIN_LAST_NAME_LENGTH,
                message: t("common.lastNameRules.minLength", {
                  minLength: Constants.GLOBALS.MIN_LAST_NAME_LENGTH,
                }),
              },
              {
                max: Constants.GLOBALS.MAX_LAST_NAME_LENGTH,
                message: t("common.lastNameRules.maxLength", {
                  maxLength: Constants.GLOBALS.MAX_LAST_NAME_LENGTH,
                }),
              },
            ]}
          >
            <Input
              placeholder={t("common.lastName")}
              maxLength={Constants.GLOBALS.MAX_LAST_NAME_LENGTH}
            />
          </Form.Item>

          <Form.Item<FieldType>
            label={t("common.email")}
            name="email"
            rules={[
              {
                required: true,
                message: t("common.emailRules.valid"),
                type: "email",
              },
            ]}
          >
            <Input placeholder="Email" />
          </Form.Item>

          <Form.Item<FieldType>
            label={t("common.password")}
            name="password"
            rules={[
              { required: true, message: t("common.passwordRules.required") },
              {
                min: Constants.GLOBALS.MIN_PASSWORD_LENGTH,
                message: t("common.passwordRules.minLength", {
                  minLength: Constants.GLOBALS.MIN_PASSWORD_LENGTH,
                }),
              },
              {
                max: Constants.GLOBALS.MAX_PASSWORD_LENGTH,
                message: t("common.passwordRules.maxLength", {
                  maxLength: Constants.GLOBALS.MAX_PASSWORD_LENGTH,
                }),
              },
            ]}
          >
            <Input.Password
              placeholder={t("common.password")}
              maxLength={Constants.GLOBALS.MAX_PASSWORD_LENGTH}
            />
          </Form.Item>

          <Form.Item name="roleId" label="Role">
            <Select>
              <Select.Option value="d0f0fa0d-3f3b-438b-a76f-7febeb8aab57">
                {tmpRoleNames["d0f0fa0d-3f3b-438b-a76f-7febeb8aab57"]}
              </Select.Option>
              <Select.Option value="b7359e12-359e-423b-b39c-f0d4069adebc">
                {tmpRoleNames["b7359e12-359e-423b-b39c-f0d4069adebc"]}
              </Select.Option>
              <Select.Option value="a1f084ad-d501-4015-b326-4c5c46fd1c5e">
                {tmpRoleNames["a1f084ad-d501-4015-b326-4c5c46fd1c5e"]}
              </Select.Option>
            </Select>
          </Form.Item>

          <Flex justify="end">
            <Button
              type="primary"
              icon={<PlusOutlined />}
              htmlType="submit"
              loading={isLoading}
            >
              {t("teamCreateUser.buttonCreateUser")}
            </Button>
          </Flex>
        </Form>
      </MyMiddleCard>
    </>
  );
}

/*
<Form.Item<FieldType>
      name="sendInvitationEmail"
      valuePropName="checked"
    >
      <Checkbox>Send an invitation email to the user</Checkbox>
    </Form.Item>
*/
